/*
 * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fnmatch.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_File.h>
#include <Ecore_Str.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <musicbrainz/queries.h>
#include <musicbrainz/mb_c.h>

#include "enna.h"
#include "enna_module.h"
#include "enna_list.h"
#include "enna_config.h"
#include "enna_mediaplayer.h"


#define SMART_NAME "enna_cdda"
#define API_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if ((!obj) || (!sd) || (evas_object_type_get(obj) && strcmp(evas_object_type_get(obj), SMART_NAME)))
#define INTERNAL_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if (!sd) return;
typedef struct _E_Smart_Data E_Smart_Data;

struct _E_Smart_Data
{
  Evas_Coord x, y, w, h;
  Evas_Object *edje;
  Evas_Object *obj;
  Evas_Object *switcher;
  Evas_Object *o_files;
  void (*exe_track_cb) (void *data1, void *data2, int selected);
  void *data1;
  void *data2;
  char *device;
  int nb_tracks;
  void (*exit_cb) (void *data1, void *data2);
  void *exit_data1;
  void *exit_data2;
};

/* local subsystem functions */

static void _switch_objects (E_Smart_Data * sd, Evas_Object * obj1,
			     Evas_Object * obj2);
static void _append_cdda_tracks (E_Smart_Data * sd, char *device);
static void _track_selected_cb (void *data, void *data2);
static void _e_smart_reconfigure (E_Smart_Data * sd);
static void _e_smart_add (Evas_Object * obj);
static void _e_smart_del (Evas_Object * obj);
static void _e_smart_move (Evas_Object * obj, Evas_Coord x, Evas_Coord y);
static void _e_smart_resize (Evas_Object * obj, Evas_Coord w, Evas_Coord h);
static void _e_smart_show (Evas_Object * obj);
static void _e_smart_hide (Evas_Object * obj);
static void _e_smart_color_set (Evas_Object * obj, int r, int g, int b,
				int a);
static void _e_smart_clip_set (Evas_Object * obj, Evas_Object * clip);
static void _e_smart_clip_unset (Evas_Object * obj);
static void _e_smart_init (void);

/* local subsystem globals */
static Evas_Smart *_e_smart = NULL;

/* externally accessible functions */
EAPI Evas_Object *
enna_cdda_add (Evas * evas)
{
  _e_smart_init ();
  return evas_object_smart_add (evas, _e_smart);
}

EAPI void
enna_cdda_process_event (Evas_Object * obj, enna_event event)
{
  API_ENTRY return;

  if (event == enna_event_escape)
    {
      if (sd->exit_cb)
	sd->exit_cb (sd->exit_data1, sd->exit_data2);
    }
  else
    enna_list_process_event (sd->o_files, event);
}

EAPI int
enna_cdda_exe_track_cb_set (Evas_Object * obj,
			    void (exe_track_cb) (void *data1, void *data2,
						 int selected), void *data1,
			    void *data2)
{
  API_ENTRY return 0;
  sd->exe_track_cb = exe_track_cb;
  sd->data1 = data1;
  sd->data2 = data2;
  return 1;
}

EAPI int
enna_cdda_exit_cb_set (Evas_Object * obj,
		       void (exit_cb) (void *data1, void *data2), void *data1,
		       void *data2)
{
  API_ENTRY return 0;
  sd->exit_cb = exit_cb;
  sd->exit_data1 = data1;
  sd->exit_data2 = data2;
  return 1;
}


EAPI void
enna_cdda_device_set (Evas_Object * obj, char *device)
{
  API_ENTRY return;

  sd->device = strdup (device);
  _append_cdda_tracks (sd, sd->device);
}

EAPI char *
enna_cdda_device_get (Evas_Object * obj)
{
  API_ENTRY return NULL;

  return sd->device;
}

EAPI int
enna_cdda_nbtracks_get (Evas_Object * obj)
{
  API_ENTRY return 0;

  return sd->nb_tracks;
}

static void
_append_cdda_tracks (E_Smart_Data * sd, char *device)
{
  int nb_tracks = 0;
  char data[4096];
  char temp[4096];
  char *artist;
  char *album;
  char *title;
  char *cdindex;
  int j;
  musicbrainz_t mb;

  mb = mb_New ();
  dbg ("device : %s\n", device);
  /* MusicBrainz Stuff */
  mb_SetDevice (mb, device);
  mb_UseUTF8 (mb, TRUE);

  if (!mb_Query (mb, MBQ_GetCDTOC))
    {
      char message[255];
      mb_GetQueryError (mb, message, 255);
      dbg ("Cannot read CD: %s\n", message);
      return;
    }

  nb_tracks = mb_GetResultInt (mb, MBE_TOCGetLastTrack);
  dbg ("Number of Tracks : %d\n", nb_tracks);

  if (!mb_GetResultData (mb, MBE_TOCGetCDIndexId, data, sizeof (data)))
    {
      mb_GetQueryError (mb, data, sizeof (data));
      dbg ("This CD could not be queried: %s\n", data);
      return;
    }
  cdindex = strdup (data);
  dbg ("cd index %s\n", cdindex);
  if (!mb_Query (mb, MBQ_GetCDInfo))
    {
      mb_GetQueryError (mb, data, sizeof (data));
      dbg ("This CD could not be queried: %s\n", data);
      return;
    }

  //nb_albums = mb_GetResultInt(mb, MBE_GetNumAlbums);
  //dbg("Number of ALbums : %d\n", nb_albums);
  //if (nb_albums >=1)
  {
    int num_tracks;

    mb_Select1 (mb, MBS_SelectAlbum, 1);
    if (mb_GetResultData (mb, MBE_AlbumGetAlbumId, data, sizeof (data)))
      mb_GetIDFromURL (mb, data, data, sizeof (data));
    if (mb_GetResultData (mb, MBE_AlbumGetAlbumArtistId, data, sizeof (data)))
      {
	mb_GetIDFromURL (mb, data, data, sizeof (data));
	if (strncasecmp (MBI_VARIOUS_ARTIST_ID, data, 64) == 0)
	  dbg ("Various Artist\n");
	else
	  {
	    if (data
		&& mb_GetResultData1 (mb, MBE_AlbumGetArtistName, data,
				      sizeof (data), 1))
	      artist = strdup (data);
	    else
	      {
		sprintf (data, "%s", "Artiste Inconnu");
		artist = strdup (data);
	      }
	  }
      }
    if (mb_GetResultData (mb, MBE_AlbumGetAlbumName, data, sizeof (data)))
      {
	dbg ("Album Name : %s\n", data);
	album = strdup (data);
      }
    else
      {
	sprintf (data, "%s", "Album Inconnu");
	dbg ("Unknown Album \n");
	album = strdup (data);
      }
    num_tracks = mb_GetResultInt (mb, MBE_AlbumGetNumTracks);
    if (!num_tracks)
      num_tracks = nb_tracks;

    dbg ("num_tracks = %d\n", num_tracks);
    sd->o_files = enna_list_add (evas_object_evas_get (sd->edje));
    evas_object_show (sd->o_files);
    sd->nb_tracks = num_tracks;
    for (j = 1; j <= num_tracks; j++)
      {
	int *id;
	Evas_Object *icon;

	id = malloc (1);

	if (mb_GetResultData1
	    (mb, MBE_AlbumGetTrackId, data, sizeof (data), j))
	  mb_GetIDFromURL (mb, data, data, sizeof (data));
	if (mb_GetResultData1
	    (mb, MBE_AlbumGetArtistId, data, sizeof (data), j))
	  mb_GetIDFromURL (mb, data, data, sizeof (data));
	if (mb_GetResultData1
	    (mb, MBE_AlbumGetTrackName, data, sizeof (data), j))
	  {
	    dbg ("Track Name : %s\n", data);
	    title = strdup (data);
	  }
	else
	  {
	    sprintf (data, "%s", "Piste Inconnu");
	    dbg ("Unknown Track \n");
	    title = strdup (data);
	  }

	*id = j;

	sprintf (temp, "%d - %s", j, title);
	icon = edje_object_add (evas_object_evas_get (sd->o_files));
	edje_object_file_set (icon, enna_config_theme_get (), "icon_music");
	evas_object_show (icon);
	enna_list_append (sd->o_files, icon, temp, 0, _track_selected_cb,
			  NULL, sd, id);
      }
    _switch_objects (sd, NULL, sd->o_files);
    enna_list_selected_set (sd->o_files, 0);
  }
  dbg ("ok\n");
}

static void
_track_selected_cb (void *data, void *data2)
{
  E_Smart_Data *sd;
  int *id;
  sd = (E_Smart_Data *) data;

  id = data2;

  dbg ("ID = %d\n", *id);

  if (sd->exe_track_cb)
    sd->exe_track_cb (sd->data1, sd->data2, *id);
}

static void
_switch_objects (E_Smart_Data * sd, Evas_Object * obj1, Evas_Object * obj2)
{
  if (!sd || (!obj1 && !obj2))
    return;

  if ((obj1 && obj2))
    {
      edje_object_part_unswallow (sd->switcher, obj1);
      edje_object_part_swallow (sd->switcher, "enna.switcher.swallow2", obj1);
      edje_object_signal_emit (sd->switcher, "enna,swallow2,default,now",
			       "enna");
      edje_object_signal_emit (sd->switcher, "enna,swallow1,state1,now",
			       "enna");
      edje_object_part_swallow (sd->switcher, "enna.switcher.swallow1", obj2);
      edje_object_signal_emit (sd->switcher, "enna,swallow1,default", "enna");
      edje_object_signal_emit (sd->switcher, "enna,swallow2,state2", "enna");
      evas_object_hide (obj1);
    }
  else if (!obj2)
    {
      edje_object_part_unswallow (sd->switcher, obj1);
      edje_object_part_swallow (sd->switcher, "enna.switcher.swallow2", obj1);
      edje_object_signal_emit (sd->switcher, "enna,swallow2,default,now",
			       "enna");
      edje_object_signal_emit (sd->switcher, "enna,swallow2,state2", "enna");
    }
  else if (!obj1)
    {
      edje_object_signal_emit (sd->switcher, "enna,swallow1,state1,now",
			       "enna");
      edje_object_part_swallow (sd->switcher, "enna.switcher.swallow1", obj2);
      edje_object_signal_emit (sd->switcher, "enna,swallow1,default", "enna");
      evas_object_show (obj2);
    }
}



static void
_e_smart_reconfigure (E_Smart_Data * sd)
{
  evas_object_move (sd->edje, sd->x, sd->y);
  evas_object_resize (sd->edje, sd->w, sd->h);
}

static void
_e_smart_add (Evas_Object * obj)
{
  E_Smart_Data *sd;
  sd = calloc (1, sizeof (E_Smart_Data));
  if (!sd)
    return;
  evas_object_smart_data_set (obj, sd);

  sd->obj = obj;
  sd->x = 0;
  sd->y = 0;
  sd->w = 0;
  sd->h = 0;

  sd->edje = edje_object_add (evas_object_evas_get (obj));
  edje_object_file_set (sd->edje, enna_config_theme_get (), "enna/cdda");
  evas_object_smart_member_add (sd->edje, obj);
  evas_object_show (sd->edje);

  sd->switcher = edje_object_add (evas_object_evas_get (obj));
  edje_object_file_set (sd->switcher, enna_config_theme_get (),
			"enna/switcher");
  evas_object_smart_member_add (sd->switcher, obj);
  evas_object_show (sd->switcher);
  edje_object_part_swallow (sd->edje, "enna.switcher.swallow", sd->switcher);
  sd->o_files = NULL;
  sd->exe_track_cb = NULL;
  sd->data1 = NULL;
  sd->data2 = NULL;
  sd->device = NULL;
  sd->nb_tracks = 0;
  evas_object_propagate_events_set (obj, 0);
}

static void
_e_smart_del (Evas_Object * obj)
{
  INTERNAL_ENTRY;
  if (sd->device)
    free (sd->device);
  evas_object_del (sd->o_files);
  evas_object_del (sd->switcher);
  evas_object_del (sd->edje);
  free (sd);
}

static void
_e_smart_move (Evas_Object * obj, Evas_Coord x, Evas_Coord y)
{
  INTERNAL_ENTRY;
  if ((sd->x == x) && (sd->y == y))
    return;
  sd->x = x;
  sd->y = y;
  _e_smart_reconfigure (sd);
}

static void
_e_smart_resize (Evas_Object * obj, Evas_Coord w, Evas_Coord h)
{
  INTERNAL_ENTRY;

  if ((sd->w == w) && (sd->h == h))
    return;
  sd->w = w;
  sd->h = h;
  _e_smart_reconfigure (sd);

}

static void
_e_smart_show (Evas_Object * obj)
{
  INTERNAL_ENTRY;
  evas_object_show (sd->edje);
}

static void
_e_smart_hide (Evas_Object * obj)
{
  INTERNAL_ENTRY;
  evas_object_hide (sd->edje);
}

static void
_e_smart_color_set (Evas_Object * obj, int r, int g, int b, int a)
{
  INTERNAL_ENTRY;
  evas_object_color_set (sd->edje, r, g, b, a);
}

static void
_e_smart_clip_set (Evas_Object * obj, Evas_Object * clip)
{
  INTERNAL_ENTRY;
  evas_object_clip_set (sd->edje, clip);
}

static void
_e_smart_clip_unset (Evas_Object * obj)
{
  INTERNAL_ENTRY;
  evas_object_clip_unset (sd->edje);
}

/* never need to touch this */

static void
_e_smart_init (void)
{
  if (_e_smart)
    return;
  static const Evas_Smart_Class sc =
    {
      SMART_NAME,
      EVAS_SMART_CLASS_VERSION,
      _e_smart_add,
      _e_smart_del,
      _e_smart_move,
      _e_smart_resize,
      _e_smart_show,
      _e_smart_hide,
      _e_smart_color_set,
      _e_smart_clip_set,
      _e_smart_clip_unset,
      NULL
    };
   _e_smart = evas_smart_class_new(&sc);
}
