#include "enna_metadata.h"

#ifndef PATH_MAX
#define PATH_MAX 4096
#endif

typedef struct _Hash_Data Hash_Data;


struct _File_Data
{
	char *sha;
};

struct _Hash_Data
{
	char *name;
	Evas_Hash *sha_hash;
};

static Eet_Data_Descriptor *edd;
static Eet_Data_Descriptor *edd_hash_sha;
static Evas_Hash *sha_hash = NULL;
static Eet_File *ef;

#ifdef WORDS_BIGENDIAN
# define SWAP(n) (n)
#else
# define SWAP(n) \
(((n) << 24) | (((n) & 0xff00) << 8) | (((n) >> 8) & 0xff00) | ((n) >> 24))
#endif

#define f1(X,Y,Z) (Z ^ (X & (Y ^ Z)))
#define f2(X,Y,Z) (X ^ Y ^ Z)
#define f3(X,Y,Z) ((X & Y) | (Z & (X | Y)))

#define rol1(x) (x<<1) | ((x>>31) & 1)
#define rol5(x) ((x<<5) | ((x>>27) & 0x1f))
#define rol30(x) (x<<30) | ((x>>2) & 0x3fffffff)



#define AMAZON_ID "0HMNNC83SA7RW4YG45R2"


static xmlNode *
_parse_xml (xmlNode * root, char *node_name)
{
	xmlNode *n, *children;
	
	for (n = root; n; n = n->next)
	{
		if (((n->type == XML_ELEMENT_NODE)
			 && (xmlStrcmp (n->name, (xmlChar *) node_name)) == 0))
		{
			return n;
		}
		if ((children = _parse_xml (n->children, node_name)))
			return children;
	}
	
	return NULL;
}

static char *
_get_cover_file (char *file)
{
	
	xmlDocPtr doc;
	xmlNode *node;
	xmlNode *root = NULL;
	
	doc = xmlParseFile (file);
	
	
	if (doc == NULL)
	{
		dbg ("File is not an XML file\n");
		return NULL;
	}
	
	root = xmlDocGetRootElement (doc);
	
	if (!(node = _parse_xml (root, "LargeImage")))
	{
		if (!(node = _parse_xml (root, "MediumImage")))
			node = _parse_xml (root, "SmallImage");
	}
	
	if (!(node = _parse_xml (node, "URL")))
		return NULL;
	else
		return strdup ((char *) xmlNodeGetContent (node));
}

static char *
_get_asin (char *file, char *node_name)
{
	
	xmlDocPtr doc;
	xmlNode *node;
	doc = xmlParseFile (file);
	xmlNode *root = NULL;
	
	if (doc == NULL)
		dbg ("File is not an XML file\n");
	
	
	root = xmlDocGetRootElement (doc);
	
	if ((node = _parse_xml (root, node_name)))
		return (char *) xmlNodeGetContent (node);
	else
	{
		dbg ("Unable to get ASIN in xml file;\n");
		return NULL;
	}
	
}


static void
_build_cover_request (char *amazonid, char *asin, char request[1024])
{
	sprintf (request,
			 "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&SubscriptionId=%s&Operation=ItemLookup&ItemId=%s&ResponseGroup=Images",
			 amazonid, asin);
}

static void
_build_request (char *amazonid, char *keywords, char request[1024])
{
	sprintf (request,
			 "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&SubscriptionId=%s&Operation=ItemSearch&Keywords=%s&SearchIndex=Music",
			 amazonid, keywords);
}

static CURLcode
_download_http_file (char *file, char *url)
{
	CURL *curl;
	CURLcode res;
	
	FILE *fp = fopen (file, "w");
	
	/* FIXME : Get permissions on file, if cannot write in directory, try to write in $HOME/.enna/cache */
	if (!fp)
		return 0;
	
	curl = curl_easy_init ();
	if (curl)
	{
		curl_easy_setopt (curl, CURLOPT_URL, url);
		curl_easy_setopt (curl, CURLOPT_WRITEDATA, fp);
		res = curl_easy_perform (curl);
		
		if (res)
			dbg ("Unable to download file.");
		
		curl_easy_cleanup (curl);
		fclose (fp);
		return res;
	}
	else
		return 0;
}



/* This function comes from Eclair source code */
static char *
_add_uri_special_chars (const char *uri)
{
	int uri_length;
	char *new_uri;
	int i, j;
	
	if (!uri)
		return NULL;
	
	uri_length = strlen (uri);
	new_uri = malloc (uri_length * 3 + 1);
	
	for (i = 0, j = 0; i < uri_length; i++, j++)
	{
		if (uri[i] == ' ' || uri[i] == ';' || uri[i] == '&' || uri[i] == '=' ||
			uri[i] == '<' || uri[i] == '>' || uri[i] == '?' || uri[i] == '@' ||
			uri[i] == '\'' || uri[i] == '[' || uri[i] == '#' || uri[i] == '\\'
			|| uri[i] == '$' || uri[i] == ']' || uri[i] == '%' || uri[i] == '^'
			||
			/*uri[i] == '\\' || uri[i] == 'š'  || uri[i] == '+'  || uri[i] == '{'  || */
			uri[i] == ',' || uri[i] == '|' || uri[i] == '/' || uri[i] == '}' ||
			uri[i] == ':' || uri[i] == '~')
		{
			new_uri[j] = '%';
			sprintf (&new_uri[j + 1], "%2x", uri[i]);
			j += 2;
		}
		else
			new_uri[j] = uri[i];
	}
	new_uri[j] = 0;
	
	return new_uri;
}

static char *
dirname (char *file)
{
	char *s;
	char *path;
	path = strdup (file);
	
	s = path + strlen (path) - 1;
	while (s && *s == '/')
	{
		*s = '\0';
		s = path + strlen (path) - 1;
	}
	s = strrchr (path, '/');
	if (s && *s)
		*s = '\0';
	
	if (ecore_str_has_prefix (file, "file://"))
		path = &path[7];
	
	return path;
}

static void
metadata_cover_get (char *filename, Enna_Metadata * metadata)
{
	Ecore_List *files;
	char *list_item;
	char request[4096];
	char keywords[4096];
	char *asin;
	char *image;
	char tmp[4096];
	char *path;
	
	if (!metadata->artist || !metadata->album)
		return;
	
	path = dirname (filename);
	
	sprintf (tmp, "%s/%s.jpg", path, metadata->album);
	if (ecore_file_exists (tmp))
	{
		metadata->cover_filename = strdup (tmp);
		return;
	}
	else
		return;
	
	sprintf (keywords, "%s %s", metadata->artist, metadata->album);
	_build_request (AMAZON_ID, _add_uri_special_chars (keywords), request);
	dbg ("URL : %s\n", request);
	
	sprintf (tmp, "%s/%s-%s.xml", path, metadata->artist, metadata->album);
	_download_http_file (tmp, request);
	
	asin = _get_asin (tmp, "ASIN");
	if (asin)
	{
		_build_cover_request (AMAZON_ID, asin, request);
		dbg ("URL : %s\n", request);
	}
	else
	{
		ecore_file_recursive_rm (tmp);
		files = ecore_file_ls (path);
		list_item = ecore_list_first_goto (files);
		while ((list_item = (char *) ecore_list_next (files)) != NULL)
		{
			sprintf (tmp, "%s/%s", path, list_item);
			if (!ecore_file_is_dir (tmp))
			{
				if (ecore_str_has_suffix (tmp, ".png")
					|| ecore_str_has_suffix (tmp, ".jpg"))
				{
					metadata->cover_filename = strdup (tmp);
					return;
				}
			}
		}
		return;
	}
	_download_http_file (tmp, request);
	image = _get_cover_file (tmp);
	ecore_file_recursive_rm (tmp);
	if (image)
	{
		sprintf (tmp, "%s/%s.jpg", path, metadata->album);
		_download_http_file (tmp, image);
		metadata->cover_filename = strdup (tmp);
	}
}

static Enna_Metadata *
metadata_new ()
{
	Enna_Metadata *metadata;
	
	metadata = calloc (1, sizeof (Enna_Metadata));
	
	return metadata;
}


static int
metadata_fs_get (char *filename, Enna_Metadata * metadata)
{
	struct stat infos;
	
	stat (filename, &infos);
	metadata->filename = strdup (filename);
	metadata->size = infos.st_size;
	
	return 1;
	
}

static int
metadata_id3tag_get (char *filename, Enna_Metadata * metadata)
{
	TagLib_File *tag_file;
	TagLib_Tag *tag;
	TagLib_AudioProperties *prop;
	
	if (!filename)
		return 0;
	
	tag_file = taglib_file_new (filename);
	if (!tag_file)
		return 0;
	
	tag = taglib_file_tag (tag_file);
	if (tag)
	{
		metadata->title = taglib_tag_title (tag);
		metadata->album = taglib_tag_album (tag);
		metadata->artist = taglib_tag_artist (tag);
		metadata->comment = taglib_tag_comment (tag);
		metadata->genre = taglib_tag_genre (tag);
		metadata->year = taglib_tag_year (tag);
		metadata->track_nbr = taglib_tag_track (tag);
	}
	
	prop = (TagLib_AudioProperties *) taglib_file_audioproperties (tag_file);
	if (prop)
	{
		metadata->duration = taglib_audioproperties_length (prop);
		metadata->bitrate = taglib_audioproperties_bitrate (prop);
		metadata->samplerate = taglib_audioproperties_samplerate (prop);
		metadata->channels = taglib_audioproperties_channels (prop);
	}
	return 1;
}


static Evas_Bool
_hash_print (Evas_Hash * hash, const char *key, void *data, void *fdata)
{
#if 0
	dbg ("--\n");
	dbg ("%s %s\n", key,
		 ((Enna_Metadata *) data)->title ? ((Enna_Metadata *) data)->
		 title : "Unknown");
	dbg ("%s %s\n", key,
		 ((Enna_Metadata *) data)->album ? ((Enna_Metadata *) data)->
		 album : "Unknown");
	dbg ("%s %s\n", key,
		 ((Enna_Metadata *) data)->artist ? ((Enna_Metadata *) data)->
		 artist : "Unknown");
	dbg ("%s %s\n", key,
		 ((Enna_Metadata *) data)->genre ? ((Enna_Metadata *) data)->
		 genre : "Unknown");
	dbg ("%s %s\n", key,
		 ((Enna_Metadata *) data)->filetype ? ((Enna_Metadata *) data)->
		 filetype : "Unknown");
	dbg ("%s %s\n", key,
		 ((Enna_Metadata *) data)->comment ? ((Enna_Metadata *) data)->
		 comment : "Unknown");
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->id);
	dbg ("%s %d MB\n", key, ((Enna_Metadata *) data)->size / (1024 * 1024));
	dbg ("%s %d seconds\n", key, ((Enna_Metadata *) data)->duration);
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->cds);
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->cd_nbr);
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->track_nbr);
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->tracks);
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->bitrate);
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->samplerate);
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->year);
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->play_count);
	dbg ("%s %d\n", key, ((Enna_Metadata *) data)->channels);
	dbg ("%s %s\n", key, ((Enna_Metadata *) data)->cover_filename);
	
	dbg ("--\n");
#endif
		return 1;
}

static Eet_Data_Descriptor *
metadata_edd_descriptor_create ()
{
	Enna_Metadata metadata;
	Eet_Data_Descriptor *edd;
	
	edd_hash_sha = eet_data_descriptor_new ("metadata", sizeof (metadata),
											(void *) evas_list_next,
											(void *) evas_list_append,
											(void *) evas_list_data,
											(void *) evas_list_free,
											(void *) evas_hash_foreach,
											(void *) evas_hash_add,
											(void *) evas_hash_free);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "title", title,
								   EET_T_STRING);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "artist",
								   artist, EET_T_STRING);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "album", album,
								   EET_T_STRING);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "genre", genre,
								   EET_T_STRING);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "filetype",
								   filetype, EET_T_STRING);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "comment",
								   comment, EET_T_STRING);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "id", id,
								   EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "size", size,
								   EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "duration",
								   duration, EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "cd_nbr",
								   cd_nbr, EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "cds", cds,
								   EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "track_nbr",
								   track_nbr, EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "tracks",
								   tracks, EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "bitrate",
								   bitrate, EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "samplerate",
								   samplerate, EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "year", year,
								   EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "play_count",
								   play_count, EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "channels",
								   channels, EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata, "id", id,
								   EET_T_INT);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd_hash_sha, Enna_Metadata,
								   "cover_filename", cover_filename,
								   EET_T_STRING);
	
	
	edd = eet_data_descriptor_new ("hash_data", sizeof (Hash_Data),
								   (void *) evas_list_next,
								   (void *) evas_list_append,
								   (void *) evas_list_data,
								   (void *) evas_list_free,
								   (void *) evas_hash_foreach,
								   (void *) evas_hash_add,
								   (void *) evas_hash_free);
	EET_DATA_DESCRIPTOR_ADD_BASIC (edd, Hash_Data, "name", name, EET_T_STRING);
	EET_DATA_DESCRIPTOR_ADD_HASH (edd, Hash_Data, "sha_hash", sha_hash,
								  edd_hash_sha);
	
	return edd;
	
}

static Enna_Metadata *
_process_file (char *filename)
{
	
	Enna_Metadata *metadata;
	
	/* Read file hash to find filename */
	metadata = evas_hash_find (sha_hash, filename);
	if (metadata)
	{
		/* filename found in file hash */
		return metadata;
	}
	else
	{
		/* File exists in file hash */
		metadata = metadata_new ();
		metadata_id3tag_get (filename, metadata);
		metadata_fs_get (filename, metadata);
		metadata_cover_get (filename, metadata);
		
		sha_hash = evas_hash_add (sha_hash, filename, metadata);
		return metadata;
	}
	
}



static void
metadata_scan_directory (char *path)
{
	Ecore_List *files = NULL;
	char *filename = NULL;
	char dir[PATH_MAX];
	
	files = ecore_file_ls (path);
	filename = ecore_list_first_goto (files);
	
	while ((filename = (char *) ecore_list_next (files)) != NULL)
	{
		sprintf (dir, "%s/%s", path, filename);
		if (filename[0] == '.')
			continue;
		else if (ecore_file_is_dir (dir))
		{
			metadata_scan_directory (strdup (dir));
		}
		else if (ecore_str_has_suffix (filename, ".mp3"))
		{
			_process_file (dir);
		}
	}
}

EAPI Enna_Metadata *
enna_metadata_get (char *filename)
{
	Enna_Metadata *metadata;
	
	metadata = _process_file (filename);
	_hash_print (NULL, "", metadata, NULL);
	
	return metadata;
}


EAPI void
enna_metadata_init (void)
{
	Hash_Data *eet_data;
	void *data;
	int size;
	eet_init ();
	char tmp[PATH_MAX];
	
	/* Create eet data descriptor */
	edd = metadata_edd_descriptor_create ();
	/* Try to open db.eet */
	
	sprintf (tmp, "%s/.enna", enna_util_user_home_get ());
	if (!ecore_file_is_dir (tmp))
		ecore_file_mkdir (tmp);
	
	sprintf (tmp, "%s/.enna/music_db.eet", enna_util_user_home_get ());
	
	/* open eet file and decode it */
	ef = eet_open (tmp, EET_FILE_MODE_READ);
	if (!ef)
	{
		dbg ("Unable to open %s\n", tmp);
	}
	else
	{
		data = eet_read (ef, "data", &size);
		eet_close (ef);
		eet_data = eet_data_descriptor_decode (edd, data, size);
	}
}

EAPI void
enna_metadata_shutdown (void)
{
	Hash_Data eet_data;
	void *data;
	int size;
	char tmp[PATH_MAX];
	
	eet_data.name = "Hash";
	
	data = eet_data_descriptor_encode (edd, &eet_data, &size);
	sprintf (tmp, "%s/.enna/music_db.eet", enna_util_user_home_get ());
	ef = eet_open (tmp, EET_FILE_MODE_WRITE);
	if (ef)
	{
		eet_write (ef, "data", data, size, 0);
		eet_close (ef);
	}
	
	eet_data_descriptor_free (edd);
	
	eet_shutdown ();
	
	
}
