#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include "enna.h"
#include "enna_event.h"
#include "enna_socket.h"
#include "enna_mplayer.h"
#include "enna_config.h"

struct event_strings
{
  enna_event evt;
  char *command;
} event_to_mplayer_cmds[] =
{
  {
  enna_event_stop, "quit\n"},
  {
  enna_event_pause, "pause\n"},
  {
  enna_event_play, "pause\n"},
  {
  enna_event_mute, "mute\n"},
  {
  enna_event_rewind, "seek -20\n"},
  {
  enna_event_fastforward, "seek +20\n"},
  {
  enna_event_language, "switch_audio\n"},
  {
  enna_event_none, ""}
};

EAPI Enna_Mplayer *
enna_mplayer_init (void)
{
  Enna_Mplayer *mplayer;

  mplayer = malloc (sizeof (Enna_Mplayer));

  return mplayer;
}

EAPI void
enna_mplayer_process_event (Enna_Mplayer * mplayer, enna_event event)
{
  int i = 0;
  // First look if we are having a simple command
  while (event_to_mplayer_cmds[i].evt != enna_event_none)
    {
      if (event_to_mplayer_cmds[i].evt == event)
	ecore_exe_send (mplayer->exe, event_to_mplayer_cmds[i].command,
			strlen (event_to_mplayer_cmds[i].command));
      i++;
    }

  if (event == enna_event_aspect)
    {
      // Simply switch from pre-calculated aspect ratio, one after an other
      static int current_ratio = 0;
      static float ratios[] = { 1, 1.33, 1.44, 1.66 };
      char buf[1024];

      if (current_ratio == 3)
	current_ratio = 0;
      else
	current_ratio++;
      snprintf (buf, 1024, "switch_ratio %.2f\n", ratios[current_ratio]);
      ecore_exe_send (mplayer->exe, buf, strlen (buf));
    }
}

EAPI int
enna_mplayer_exited (Enna_Mplayer * mplayer)
{
  return 1;
}

int
_mplayer_data_received_cb (void *data, int type, void *event)
{
  Ecore_Exe_Event_Data *ev;

  ev = event;
  /*
     if (ev->lines)
     for (i = 0; ev->lines[i].line != NULL; i++)
     dbg("[Mplayer] -- %s\n", ev->lines[i].line);
   */
  return 1;
}

EAPI void
enna_mplayer_run (Enna_Mplayer * mplayer, char *filename)
{
  char tmp[4096];
  ecore_event_handler_add (ECORE_EXE_EVENT_DATA, _mplayer_data_received_cb,
			   NULL);
  snprintf (tmp, 4096, "/usr/bin/mplayer -fs -slave -quiet");

  if (enna_config_get_conf_value ("mplayer", "vo"))
    snprintf (tmp + strlen (tmp), 4096 - strlen (tmp), " -vo %s",
	      enna_config_get_conf_value ("mplayer", "vo"));

  if (enna_config_get_conf_value ("mplayer", "ao"))
    snprintf (tmp + strlen (tmp), 4096 - strlen (tmp), " -ao %s",
	      enna_config_get_conf_value ("mplayer", "ao"));

  if (enna_config_get_conf_value ("mplayer", "monitoraspect"))
    snprintf (tmp + strlen (tmp), 4096 - strlen (tmp), " -monitoraspect %s",
	      enna_config_get_conf_value ("mplayer", "monitoraspect"));

  snprintf (tmp + strlen (tmp), 4096 - strlen (tmp), " \"%s\"", filename);

  dbg ("playing with command: %s\n", tmp);

  mplayer->exe = ecore_exe_pipe_run (tmp,
				     ECORE_EXE_PIPE_WRITE |
				     ECORE_EXE_PIPE_READ |
				     ECORE_EXE_PIPE_READ_LINE_BUFFERED, NULL);
}
