/*
 * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */

#include <unistd.h>
#include <stdio.h>
#include "enna.h"
#include "enna_event.h"
#include "enna_socket.h"
#include "enna_xine.h"


#define DEFAULT_XINE_PORT 6789
#define BUF_SIZE 4096


static int
_read_xine_data (void *data, Ecore_Fd_Handler * fd_handler)
{
  struct xine *x = (struct xine *) data;
  static char buf[BUF_SIZE];
  int ret;

  while ((ret = read (x->xine_sd, buf, BUF_SIZE)) > 0)
    {
    }
  return 1;
}


static void
_connect_to_xine_server (struct xine *x)
{
  x->xine_sd = enna_connect ("localhost", DEFAULT_XINE_PORT, 1, NULL, 0);
  x->xine_handler =
    ecore_main_fd_handler_add (x->xine_sd, ECORE_FD_READ, _read_xine_data, x,
			       NULL, NULL);
}


struct dvd_event_strings
{
  enna_event evt;
  char *command;
} event_to_XINE_cmds[] =
{
  {
  enna_event_exit, "halt\n"},
  {
  enna_event_stop, "halt\n"},
  {
  enna_event_pause, "pause\n"},
  {
  enna_event_play, "play\n"},
  {
  enna_event_language, "\n"},
  {
  enna_event_mute, "\n"},
  {
  enna_event_up, "event up\n"},
  {
  enna_event_down, "event down\n"},
  {
  enna_event_left, "event left\n"},
  {
  enna_event_escape, "event menu\n"},
  {
  enna_event_right, "event right\n"},
  {
  enna_event_enter, "event select\n"},
  {
  enna_event_rewind, "seek -30\n"},
  {
  enna_event_fastforward, "seek +30\n"},
  {
  enna_event_prev, "event prev\n"},
  {
  enna_event_next, "event next\n"},
  {
  enna_event_none, ""}
};

void
enna_xine_process_event (struct xine *x, enna_event event)
{
  int i = 0;

  while (event_to_XINE_cmds[i].evt != enna_event_none)
    {
      if (event_to_XINE_cmds[i].evt == event)
	{
	  if (x->xine_sd < 0)
	    _connect_to_xine_server (x);
	  if (x->xine_sd > -1)
	    write (x->xine_sd, event_to_XINE_cmds[i].command,
		   strlen (event_to_XINE_cmds[i].command));
	}
      i++;
    }
}

int
enna_xine_exited (struct xine *x)
{
  if (x->xine_sd > -1)
    close (x->xine_sd);
  if (x->xine_handler)
    ecore_main_fd_handler_del (x->xine_handler);
  x->xine_sd = -1;
  x->xine_handler = NULL;
  return 1;
}

static int
_xine_data_received_cb (void *data, int type, void *event)
{

  return 1;
}

void
enna_run_xine (struct xine *x)
{
  char tmp[4096];

  ecore_event_handler_add (ECORE_EXE_EVENT_DATA, _xine_data_received_cb,
			   NULL);
  sprintf (tmp,
	   "/usr/bin/xine -f -g --no-splash -n --network --network-port 6789 dvd://");
  dbg ("playing with command: %s\n", tmp);

  x->xine_speed = 1;
  x->xine_exe =
    ecore_exe_pipe_run (tmp,
			ECORE_EXE_PIPE_WRITE | ECORE_EXE_PIPE_READ |
			ECORE_EXE_PIPE_READ_LINE_BUFFERED, NULL);
}
